#include "main.hpp"
#include <unicode/stringoptions.h>

void init_stringoptions(py::module &m) {
  m.attr("U_COMPARE_CODE_POINT_ORDER") = U_COMPARE_CODE_POINT_ORDER;
  m.attr("U_COMPARE_IGNORE_CASE") = U_COMPARE_IGNORE_CASE;

#if (U_ICU_VERSION_MAJOR_NUM >= 60)
  m.attr("U_EDITS_NO_RESET") = U_EDITS_NO_RESET;
#endif // (U_ICU_VERSION_MAJOR_NUM >= 60)

  m.attr("U_FOLD_CASE_DEFAULT") = U_FOLD_CASE_DEFAULT;
  m.attr("U_FOLD_CASE_EXCLUDE_SPECIAL_I") = U_FOLD_CASE_EXCLUDE_SPECIAL_I;

#if (U_ICU_VERSION_MAJOR_NUM >= 60)
  m.attr("U_OMIT_UNCHANGED_TEXT") = U_OMIT_UNCHANGED_TEXT;
  m.attr("U_TITLECASE_ADJUST_TO_CASED") = U_TITLECASE_ADJUST_TO_CASED;
#endif // (U_ICU_VERSION_MAJOR_NUM >= 60)

  m.attr("U_TITLECASE_NO_BREAK_ADJUSTMENT") = U_TITLECASE_NO_BREAK_ADJUSTMENT;
  m.attr("U_TITLECASE_NO_LOWERCASE") = U_TITLECASE_NO_LOWERCASE;

#if (U_ICU_VERSION_MAJOR_NUM >= 60)
  m.attr("U_TITLECASE_SENTENCES") = U_TITLECASE_SENTENCES;
  m.attr("U_TITLECASE_WHOLE_STRING") = U_TITLECASE_WHOLE_STRING;
#endif // (U_ICU_VERSION_MAJOR_NUM >= 60)

  m.attr("UNORM_INPUT_IS_FCD") = UNORM_INPUT_IS_FCD;
}
